import webpack, { Entry } from 'webpack'
import MemoryFS from 'memory-fs'
import {
  Externals,
  WebpackError,
  BuildPackageOptions,
  CreateEntryPointOptions,
} from '../common.types'
declare type CompilePackageArgs = {
  name: string
  externals: Externals
  entry: Entry
  debug?: boolean
  minifier: 'terser' | 'esbuild'
}
declare type CompilePackageReturn = {
  stats: webpack.Stats
  error: WebpackError
  memoryFileSystem: MemoryFS
}
declare type BuildPackageArgs = {
  name: string
  installPath: string
  externals: Externals
  options: BuildPackageOptions
}
declare const BuildUtils: {
  createEntryPoint(
    packageName: string,
    installPath: string,
    options: CreateEntryPointOptions
  ): string
  compilePackage({
    name,
    entry,
    externals,
    debug,
    minifier,
  }: CompilePackageArgs): Promise<CompilePackageReturn>
  _parseMissingModules(errors: Array<WebpackError>): string[]
  buildPackage({
    name,
    installPath,
    externals,
    options,
  }: BuildPackageArgs): Promise<{
    dependencySizes?:
      | {
          name: string
          approximateSize: number
        }[]
      | undefined
    assets: {
      name: string
      type: string
      size: number
      gzip: number
      parse:
        | {
            baseParseTime: number
            scriptParseTime: number
          }
        | {
            baseParseTime?: undefined
            scriptParseTime?: undefined
          }
        | null
    }[]
  }>
  buildPackageIgnoringMissingDeps({
    name,
    externals,
    installPath,
    options,
  }: BuildPackageArgs): Promise<
    | {
        dependencySizes?:
          | {
              name: string
              approximateSize: number
            }[]
          | undefined
        assets: {
          name: string
          type: string
          size: number
          gzip: number
          parse:
            | {
                baseParseTime: number
                scriptParseTime: number
              }
            | {
                baseParseTime?: undefined
                scriptParseTime?: undefined
              }
            | null
        }[]
      }
    | {
        dependencySizes?:
          | {
              name: string
              approximateSize: number
            }[]
          | undefined
        assets: {
          name: string
          type: string
          size: number
          gzip: number
          parse:
            | {
                baseParseTime: number
                scriptParseTime: number
              }
            | {
                baseParseTime?: undefined
                scriptParseTime?: undefined
              }
            | null
        }[]
        ignoredMissingDependencies: any
      }
  >
}
export default BuildUtils
