'use strict'
var __importDefault =
  (this && this.__importDefault) ||
  function (mod) {
    return mod && mod.__esModule ? mod : { default: mod }
  }
Object.defineProperty(exports, '__esModule', { value: true })
exports.parsePackageString = exports.getExternals = exports.exec = void 0
const child_process_1 = __importDefault(require('child_process'))
const path_1 = __importDefault(require('path'))
const builtin_modules_1 = __importDefault(require('builtin-modules'))
const fs_1 = __importDefault(require('fs'))
const os_1 = __importDefault(require('os'))
const homeDirectory = os_1.default.homedir()
function exec(command, options, timeout) {
  let timerId
  return new Promise((resolve, reject) => {
    const child = child_process_1.default.exec(
      command,
      options,
      (error, stdout, stderr) => {
        if (error) {
          reject(stderr)
        } else {
          resolve(stdout)
        }
        if (timerId) {
          clearTimeout(timerId)
        }
      }
    )
    if (timeout) {
      timerId = setTimeout(() => {
        process.kill(child.pid)
        reject(
          `Execution of ${command.substring(
            0,
            40
          )}... cancelled as it exceeded a timeout of ${timeout} ms`
        )
      }, timeout)
    }
  })
}
exports.exec = exec
/**
 * Gets external peerDeps that shouldn't be a
 * part of the build in a regex format -
 * /(^dep-a$|^dep-a\/|^dep-b$|^dep-b\/)\//
 */
function getExternals(packageName, installPath) {
  const packageJSONPath = path_1.default.join(
    installPath,
    'node_modules',
    packageName,
    'package.json'
  )
  const packageJSON = require(packageJSONPath)
  const dependencies = Object.keys(packageJSON.dependencies || {})
  const peerDependencies = Object.keys(packageJSON.peerDependencies || {})
  // All packages with name same as a built-in node module, but
  // haven't explicitly been added as an npm dependency or aren't the package itself
  // are externals
  const builtInExternals = builtin_modules_1.default.filter(
    mod => !dependencies.includes(mod) && mod !== packageName
  )
  return {
    externalPackages: peerDependencies,
    externalBuiltIns: builtInExternals,
  }
}
exports.getExternals = getExternals
function expandTilde(pathString) {
  return homeDirectory
    ? pathString.replace(/^~(?=$|\/|\\)/, homeDirectory)
    : pathString
}
function isLocalPackageString(packageString) {
  const packageJsonPath = path_1.default.resolve(packageString, 'package.json')
  try {
    if (fs_1.default.existsSync(packageJsonPath)) {
      return true
    }
  } catch (err) {
    return false
  }
}
function isScopedPackageString(packageString) {
  return packageString.startsWith('@')
}
function parseLocalPackageString(packageString) {
  const fullPath = path_1.default.resolve(packageString, 'package.json')
  const packageJSON = require(fullPath)
  return {
    name: packageJSON.name,
    version: packageJSON.version,
    scoped: packageJSON.name.startsWith('@'),
    normalPath: packageString,
    isLocal: true,
  }
}
function parseScopedPackageString(packageString) {
  const lastAtIndex = packageString.lastIndexOf('@')
  return {
    name:
      lastAtIndex === 0
        ? packageString
        : packageString.substring(0, lastAtIndex),
    version:
      lastAtIndex === 0 ? null : packageString.substring(lastAtIndex + 1),
    scoped: true,
  }
}
function parseUnscopedPackageString(packageString) {
  const lastAtIndex = packageString.lastIndexOf('@')
  return {
    name:
      lastAtIndex === -1
        ? packageString
        : packageString.substring(0, lastAtIndex),
    version:
      lastAtIndex === -1 ? null : packageString.substring(lastAtIndex + 1),
    scoped: false,
  }
}
function parsePackageString(packageString) {
  const normalPackageString = expandTilde(packageString)
  if (isLocalPackageString(normalPackageString)) {
    return parseLocalPackageString(normalPackageString)
  } else if (isScopedPackageString(normalPackageString)) {
    return parseScopedPackageString(normalPackageString)
  } else {
    return parseUnscopedPackageString(normalPackageString)
  }
}
exports.parsePackageString = parsePackageString
